//--------------------------------------------------------------------------
// File : Main.cpp
// Desc : Median Filter Sample Program
// Copyright(c) 2010 Pocol.
//--------------------------------------------------------------------------

//--------------------------------------------------------------------------
// History
//--------------------------------------------------------------------------
// <---> 2010/11/07 [Pocol] VK쐬

//
// include
//
#include "framework/framework.h"
using namespace std;

//
// Global Variable
//
GLUTState Window = { 
	Vector2i(100, 100),
	Vector2i(800, 600),
	Vector2i(0, 0),
	Vector2i(0, 0),
	false,
	"Median Filter",
	NULL, 
	NULL,
	NULL,
	NULL,
	0.0f,
	0.0, 0.0,
	0.0, 0.0,
	0,
};
ThirdPersonCamera   g_Camera;
Vector4f            g_lightPosition(3.0f, 3.0f, 3.0f, 1.0f);
Color4f             g_lightAmbient(0.0f, 0.0f, 0.0f, 1.0f);
Color4f             g_lightDiffuse(1.0f, 1.0f, 1.0f, 1.0f);
AsuraUserInterface  g_UI;
AsuraTexture        g_Texture;


#if USE_CG
Matrix4f            g_World;
Matrix4f            g_View;
Matrix4f            g_Projection;
CGcontext           g_Context = NULL;
CGeffect            g_Effect = NULL;
CGtechnique         g_Technique = NULL;
CGparameter         g_hWorld = NULL;
CGparameter         g_hView = NULL;
CGparameter         g_hProjection = NULL;
CGparameter         g_hTexture = NULL;
#endif

//----------------------------------------------------------------------------------------------------
// Name : main()
// Desc : AvP[ṼGg[|Cg
//----------------------------------------------------------------------------------------------------
int main( int argc, char **argv )
{
	//@GLUTR[obN֐Ȃǂ̐ݒ
	glutInit( &argc, argv );
	glutInitWindowPosition( Window.Position.x, Window.Position.y );
	glutInitWindowSize( Window.Size.x, Window.Size.y );
	glutInitDisplayMode( GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE );
	glutCreateWindow( Window.Title );
	glutDisplayFunc( RenderFunc );
	glutReshapeFunc( ReshapeFunc );
	glutIdleFunc( IdleFunc );
	glutMouseFunc( MouseFunc );
	glutMotionFunc( MotionFunc );
	glutPassiveMotionFunc( PassiveMotionFunc );
	glutKeyboardFunc( KeyboardFunc );
	glutSpecialFunc( SpecialFunc );
	glutCreateMenu( MenuFunc );
	
	//@OpenGL
	Initialize();	

	//@C[v
	glutMainLoop();

	//@Еt
	Shutdown();

	return 0;
}

//----------------------------------------------------------------------------------------------------
// Name : Initialize()
// Desc : 
//----------------------------------------------------------------------------------------------------
void Initialize()
{
	const GLubyte *version = glGetString(GL_VERSION);
	const GLubyte *vender = glGetString(GL_VENDOR);
	const GLubyte *renderer = glGetString(GL_RENDERER);

	sprintf(Window.Version, "OpenGL %s", version);
	sprintf(Window.Vender, "%s", vender);
	sprintf(Window.Renderer, "%s", renderer);

	//@obNobt@NAF̎w
	Color4f CornflowerBlue = ColorRGBA(100, 149, 237, 255);
	glClearColor(CornflowerBlue.r, CornflowerBlue.g, CornflowerBlue.b, CornflowerBlue.a);

	//@[xeXgL
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	//@JnԂۑ
	Window.prevCount = glutGet(GLUT_ELAPSED_TIME);

	//@|bvAbvj[ǉ
	glutAddMenuEntry("ToggleFullScreen (F1)", GLUT_KEY_F1);
	glutAddMenuEntry("Exit (ESC)", '\033');
	glutAttachMenu(GLUT_MIDDLE_BUTTON);

#if USE_GLEW
	//@GLEW̏
	if ( glewInit() != GLEW_OK )
	{
		cout << "Error : GLEW Initialize false\n";
		Shutdown();
	}
#endif

	//@eNX`̃[h
	AsuraCreateTextureFromFileA( "Textures/lena_with_noise.dds", g_Texture );

#if USE_CG
	//@Cg̏
	InitializeCg();
#endif

}

//---------------------------------------------------------------------------------------------------
// Name : IdleFunc()
// Desc : AChȌ
//---------------------------------------------------------------------------------------------------
void IdleFunc()
{
	//@XV
	Update();

	//@ĕ`
	glutPostRedisplay();
}

//---------------------------------------------------------------------------------------------------
// Name : ReshapeFunc()
// Desc : TCYύX
//---------------------------------------------------------------------------------------------------
void ReshapeFunc(int x, int y)
{
	//@EBhETCYۑ
	Window.Size.x = x;
	Window.Size.y = y;

	//@TCY`FbN
	if ( Window.Size.x < 1 ) Window.Size.x = 1;
	if ( Window.Size.y < 1 ) Window.Size.y = 1;

	//@AXyNg
	float aspectRatio = (float)Window.Size.x/(float)Window.Size.y;

	//@r[|[g̐ݒ
	glViewport(0, 0, Window.Size.x, Window.Size.y);

	//@ˉes̐ݒ
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, aspectRatio, 0.01, 1000.0);
	g_Projection = CreatePerspectiveFieldOfView( DegToRad(45.0f), aspectRatio, 0.01f, 1000.0f );
}

//---------------------------------------------------------------------------------------------------
// Name : Update()
// Desc : XV
//---------------------------------------------------------------------------------------------------
void Update()
{
	sprintf( Window.FpsString, "%.3f FPS", glutGetFPS() );
}

//---------------------------------------------------------------------------------------------------
// Name : RenderFunc()
// Desc : EBhEւ̕`
//---------------------------------------------------------------------------------------------------
void RenderFunc()
{
	//@obt@NA
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	//@[hr[s̐ݒ
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	//@_̐ݒ
	g_View = CreateLookAt( Vector3f( 0.0f, 0.0f, 4.0f), Vector3f().Zero(), Vector3f().Up() );

	//@V[`
	RenderScene();

	//@GUȈ
	DoUI();

	//@_uobt@O
	glutSwapBuffers();

#if defined(DEBUG) || defined(_DEBUG)
	//@G[`FbN
	glutReportErrors();
#endif
}

//---------------------------------------------------------------------------------------------------
// Name : RenderQuad()
// Desc : ``
//---------------------------------------------------------------------------------------------------
void RenderQuad( float x, float y, float width, float height )
{
	//@lp``
	glBegin( GL_QUADS );
	glColor4f( 1.0f, 1.0f, 1.0f, 1.0f );
	glTexCoord2f( 0.0f, 0.0f );	glVertex3f( x, y + height, 0.0f );
	glTexCoord2f( 0.0f, 1.0f );	glVertex3f( x, y, 0.0f );
	glTexCoord2f( 1.0f, 1.0f );	glVertex3f( x + width, y, 0.0f );
	glTexCoord2f( 1.0f, 0.0f );	glVertex3f( x + width, y + height, 0.0f );
	glEnd();
}

//--------------------------------------------------------------------------------------------------
// Name : RenderScene()
// Desc : V[̕`
//--------------------------------------------------------------------------------------------------
void RenderScene()
{
	glPushMatrix();

	//@f[^𑗂
	cgSetMatrixParameterfr( g_hWorld, g_World );
	cgSetMatrixParameterfr( g_hView, g_View );
	cgSetMatrixParameterfr( g_hProjection, g_Projection );

	//@tB^Oʂ`
    CGpass pass = cgGetFirstPass(g_Technique);
    cgSetPassState(pass);
    RenderQuad( -2.0f, -1.0f, 2.0f, 2.0f );
 	cgResetPassState(pass);

    //@rpf[^
    pass = cgGetNextPass(pass);
    cgSetPassState(pass);
    RenderQuad( 0.0f, -1.0f, 2.0f, 2.0f );
 	cgResetPassState(pass);

 
	glPopMatrix();
}

//-------------------------------------------------------------------------------------------------
// Name : DoUI()
// Desc : GUI̕`Ə
//-------------------------------------------------------------------------------------------------
void DoUI()
{
	g_UI.Begin();

	g_UI.DoText(15, 1, Window.Version);
	g_UI.DoText(15, 16, Window.Vender);
	g_UI.DoText(15, 31, Window.Renderer);
	g_UI.DoText(15, 46, Window.FpsString);

	g_UI.End();
}

//---------------------------------------------------------------------------------------------------
// Name : MouseFunc()
// Desc : }EX
//---------------------------------------------------------------------------------------------------
void MouseFunc(int button, int state, int x, int y)
{
	g_Camera.MouseFunc(button, state, x, y);
	g_UI.Mouse(button, state, x, y);
}

//--------------------------------------------------------------------------------------------------
// Name : MotionFunc()
// Desc : }EXhbO
//-------------------------------------------------------------------------------------------------
void MotionFunc(int x, int y)
{
	g_Camera.MotionFunc(x, y);
	g_UI.Motion(x, y);
}

//--------------------------------------------------------------------------------------------------
// Name : PassiveMotionFunc()
// Desc : }EXړ
//--------------------------------------------------------------------------------------------------
void PassiveMotionFunc(int x, int y)
{
	g_UI.PassiveMotion(x, y);
}

//--------------------------------------------------------------------------------------------------
// Name : KeyboardFunc()
// Desc : L[{[h
//--------------------------------------------------------------------------------------------------
void KeyboardFunc(unsigned char key, int x, int y)
{
	switch ( key )
	{
	case '\033':
		Shutdown();
		break;

	default:
		break;
	}
}

//--------------------------------------------------------------------------------------------------
// Name : SpecialFunc()
// Desc : L[
//--------------------------------------------------------------------------------------------------
void SpecialFunc(int key, int x, int y)
{
	switch ( key )
	{
	case GLUT_KEY_F1:
		glutToggleFullScreen();
		break;

	case GLUT_KEY_F2:
		break;

	case GLUT_KEY_F3:
		break;

	case GLUT_KEY_F4:
		break;

	case GLUT_KEY_F5:
		break;

	case GLUT_KEY_F6:
		break;

	case GLUT_KEY_F7:
		break;

	case GLUT_KEY_F8:
		break;

	case GLUT_KEY_F9:
		break;

	case GLUT_KEY_F10:
		break;

	case GLUT_KEY_F11:
		break;

	case GLUT_KEY_F12:
		break;

	case GLUT_KEY_LEFT:
		break;

	case GLUT_KEY_RIGHT:
		break;

	case GLUT_KEY_UP:
		break;

	case GLUT_KEY_DOWN:
		break;

	case GLUT_KEY_PAGE_UP:
		break;

	case GLUT_KEY_PAGE_DOWN:
		break;

	case GLUT_KEY_HOME:
		break;

	case GLUT_KEY_END:
		break;

	case GLUT_KEY_INSERT:
		break;
	}
}

//--------------------------------------------------------------------------------------------------
// Name : MenuFunc()
// Desc : |bvAbvj[̏
//--------------------------------------------------------------------------------------------------
void MenuFunc(int item)
{
	if ( (1 <= item && item <= 12 ) 
		|| (100<=item && item <= 108) ) 
		SpecialFunc(item, 0, 0);
	else
		KeyboardFunc((unsigned char)item, 0, 0);
}

//--------------------------------------------------------------------------------------------------
// Name : Shutdown()
// Desc : Еt
//--------------------------------------------------------------------------------------------------
void Shutdown()
{
#if USE_CG
	ShutdownCg();
#endif

	exit(0);
}

//--------------------------------------------------------------------------------------------------
// Name : glutRenderText()
// Desc : ASCII̕`
//--------------------------------------------------------------------------------------------------
void glutRenderText(void* bitmapfont, char*text)
{
	for ( int i=0; i<(int)strlen(text); i++ )
		glutBitmapCharacter(bitmapfont, (int)text[i]);
}

//--------------------------------------------------------------------------------------------------
// Name : glutToggleFullScreen()
// Desc : tXN[ƃEBhE[h̐؂ւ
//--------------------------------------------------------------------------------------------------
void glutToggleFullScreen()
{
	Window.FullScreen = (Window.FullScreen ? false : true);
	if ( Window.FullScreen )
	{
		Window.PrevPosition.x = glutGet(GLUT_WINDOW_X);
		Window.PrevPosition.y = glutGet(GLUT_WINDOW_Y);
		Window.PrevSize.x = glutGet(GLUT_WINDOW_WIDTH);
		Window.PrevSize.y = glutGet(GLUT_WINDOW_HEIGHT);
		glutFullScreen();
	}
	else
	{
		glutPositionWindow(Window.PrevPosition.x, Window.PrevPosition.y);
		glutReshapeWindow(Window.PrevSize.x, Window.PrevSize.y);
	}
}

//---------------------------------------------------------------------------------------------------
// Name : glutGetFPS()
// Desc : FPS擾
//--------------------------------------------------------------------------------------------------
float glutGetFPS()
{
	Window.currCount = glutGet(GLUT_ELAPSED_TIME);
	Window.currTime = (Window.currCount - Window.prevCount)/1000.0;
	Window.frame++;

	//@Ō̍XV̌oߎԂZo
	double dtime = (Window.currTime - Window.prevTime);

	//@1bȏソXV
	if ( dtime >= 1.0 )
	{
		Window.FPS = (float)(Window.frame/dtime);
		Window.frame = 0;
		Window.prevTime = Window.currTime;
	}

	return Window.FPS;
}


#if USE_CG 
//---------------------------------------------------------------------------------------------------
// Name : CheckCgError()
// Desc : CgG[`FbN
//---------------------------------------------------------------------------------------------------
void CheckCgError()
{
	CGerror lastError = cgGetError();
	if ( lastError )
	{
		cout << "Error : CgG[܂\n";
		cout << cgGetErrorString(lastError) << endl;
		cout << cgGetLastListing(g_Context) << endl;
		Shutdown();
	}
}

//---------------------------------------------------------------------------------------------------
// Name : InitializeCg()
// Desc : Cg̏
//---------------------------------------------------------------------------------------------------
void InitializeCg()
{
	//@G[R[obN֐̐ݒ
	cgSetErrorCallback(CheckCgError);

	//@ReLXg쐬
	g_Context = cgCreateContext();
	cgGLRegisterStates(g_Context);

	//@GtFNg쐬
	g_Effect = cgCreateEffectFromFile(g_Context, "Shader/MedianFilter.cgfx", NULL);

	//@eNjbN擾
	g_Technique = cgGetNamedTechnique(g_Effect, "Render");

	//@`FbN
	while ( g_Technique && cgValidateTechnique(g_Technique) == CG_FALSE )
	{
		cout << "Error : Technique " << cgGetTechniqueName(g_Technique) << " did not valid. Skipping\n";
		g_Technique = cgGetNextTechnique(g_Technique);
	}
	if ( !g_Technique )
	{
		cout << "Error : No valid techniques\n";
		Shutdown();
	}

	//@p[^擾
	g_hWorld = cgGetNamedEffectParameter(g_Effect, "World");
	g_hView = cgGetNamedEffectParameter(g_Effect, "View");
	g_hProjection = cgGetNamedEffectParameter(g_Effect, "Projection");
	g_hTexture = cgGetNamedEffectParameter( g_Effect, "TextureSmp" );

	//@eNX`𑗂
	cgGLSetTextureParameter( g_hTexture, g_Texture.GetHandle() );
	cgSetSamplerState( g_hTexture );

}

//---------------------------------------------------------------------------------------------------
// Name : ShutdownCg()
// Desc : CǧЕt
//---------------------------------------------------------------------------------------------------
void ShutdownCg()
{
	if ( g_Effect )
	{
		cgDestroyEffect(g_Effect);
		g_Effect = NULL;
	}
	if ( g_Context )
	{
		cgDestroyContext(g_Context);
		g_Context = NULL;
	}
}
#endif

#if USE_GLEW
//---------------------------------------------------------------------------------------------------
// Name : CopyWindowToTexture()
// Desc : `eeNX`ɃRs[
//---------------------------------------------------------------------------------------------------
void CopyWindowToTexture(GLenum target, GLuint *handle, bool bufferClear)
{
	if ( *handle )
	{
		glBindTexture(target, *handle);
		glCopyTexSubImage2D(target, 0, 0, 0, 0, 0, Window.Size.x, Window.Size.y);
	}
	else
	{
		glGenTextures(1, handle);
		glBindTexture(target, *handle);

		glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

		glCopyTexImage2D(target, 0, GL_RGBA, 0, 0, Window.Size.x, Window.Size.y, 0);
	}

	if ( bufferClear )
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
}
#endif
