//----------------------------------------------------------------------------------
// File : AsuraInterface.cpp
//
// Asura User Interface Library
//
// Date : May 27, 2008
// Version : 1.0
// Author : Pocol
//----------------------------------------------------------------------------------

//
// include
//
#include "AsuraInterface.h"

namespace Asura
{

enum UIState
{
	UI_NDOWN_AND_OVERLAP = 1,
	UI_NDOWN_AND_NOVERLAP,
	UI_DOWN_AND_OVERLAP,
	UI_DOWN_AND_NOVERLAP,
};

//--------------------------------------------------------------------------------------------------------
// Name : CheckRnage()
// Desc : ͈̓`FbN
//--------------------------------------------------------------------------------------------------------
bool CheckRange(float val, float min, float max)
{
	if ( min <= val && val <= max ) 
		return true;
	return false;
}

//-----------------------------------------------------------------------------------------------------------
// Name : DrawRect()
// Desc : lp``
//-----------------------------------------------------------------------------------------------------------
void DrawRect(
			  float x, float y, float w, float h,
			  float r1, float g1, float b1, float a1,
			  float r2, float g2, float b2, float a2,
			  float r3, float g3, float b3, float a3,
			  float r4, float g4, float b4, float a4,
			  float depth=0.0f)
{
	glBegin(GL_QUADS);
	
	glColor4f(r1, g1, b1, a1);
	glVertex3f(x, y, depth);

	glColor4f(r2, g2, b2, a2);
	glVertex3f(x, y+h, depth);

	glColor4f(r3, g3, b3, a3);
	glVertex3f(x+w, y+h, depth);

	glColor4f(r4, g4, b4, a4);
	glVertex3f(x+w, y, depth);
	glEnd();
}

//-----------------------------------------------------------------------------------------------------------
// Name : DrawRect()
// Desc : lp``
//-----------------------------------------------------------------------------------------------------------
void DrawRect(
			  float x, float y, float w, float h,
			  float upR, float upG, float upB, float upA,
			  float dwR, float dwG, float dwB, float dwA,
			  float depth=0.0f)
{
	DrawRect(
		x, y, w, h,
		upR, upG, upB, upA,
		dwR, dwG, dwB, dwA,
		dwR, dwG, dwB, dwA,
		upR, upG, upB, upA, 
		depth);
}

//------------------------------------------------------------------------------------------------------------
// Name : DrawRect()
// Desc : lp``
//------------------------------------------------------------------------------------------------------------
void DrawRect(
			  float x, float y, float w, float h,
			  float cr, float cg, float cb, float ca, float depth=0.0f)
{
	DrawRect(
		x, y, w, h,
		cr, cg, cb, ca,
		cr, cg, cb, ca,
		depth);
}

//-----------------------------------------------------------------------------------------------------------
// Name : DrawRectLine()
// Desc : Ŏlp``
//-----------------------------------------------------------------------------------------------------------
void DrawRectLine(float x, float y, float w, float h, float depth=0.0f)
{
	glBegin(GL_LINE_STRIP);
	glVertex3f(x, y, depth);
	glVertex3f(x, y+h, depth);
	glVertex3f(x+w, y+h, depth);
	glVertex3f(x+w, y, depth);
	glVertex3f(x, y, depth);
	glEnd();
}

//---------------------------------------------------------------------------------------------------
// Name : AsuraUserInterface()
// Desc : RXgN^
//---------------------------------------------------------------------------------------------------
AsuraUserInterface::AsuraUserInterface()
{
	m_alpha = 0.75f;
	m_state = MOUSE_BUTTON_UP;
	m_stateClone = MOUSE_BUTTON_UP;
	m_cursor.Zero();
	m_delta.Zero();
	m_down.Zero();
	m_up.Zero();
	m_lightingEnable = false;
}

//----------------------------------------------------------------------------------------------------
// Name : ~AsuraUserInterface()
// Desc : fXgN^
//----------------------------------------------------------------------------------------------------
AsuraUserInterface::~AsuraUserInterface()
{
}

//----------------------------------------------------------------------------------------------------
// Name : Begin()
// Desc : [U[C^[tF[X̕`Jn
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::Begin()
{
	m_windowSize.x = glutGet( GLUT_WINDOW_WIDTH );
	m_windowSize.y = glutGet( GLUT_WINDOW_HEIGHT );

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D( 0, m_windowSize.x, m_windowSize.y, 0 );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	glLoadIdentity();

	glGetBooleanv( GL_LIGHTING, &m_lightingEnable );
	glGetBooleanv( GL_BLEND, &m_blendEnable );
	glGetIntegerv( GL_BLEND_SRC, &m_blendSrc );
	glGetIntegerv( GL_BLEND_DST, &m_blendDst );

	glDisable( GL_LIGHTING );
	glEnable( GL_BLEND );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

	glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );

}

//---------------------------------------------------------------------------------------------------
// Name : End()
// Desc : [U[C^[tF[X̕`I
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::End()
{
	if ( m_blendEnable ) glEnable( GL_BLEND );
	else glDisable( GL_BLEND );
	glBlendFunc( m_blendSrc, m_blendDst );

	if ( m_lightingEnable )	glEnable( GL_LIGHTING );

	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();
}

//----------------------------------------------------------------------------------------------------
// Name : SetAlpha()
// Desc : At@lݒ
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::SetAlpha( float alpha )
{
	m_alpha = alpha;
}

//----------------------------------------------------------------------------------------------------
// Name : Mouse()
// Desc : }EX̓͏
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::Mouse(int button, int state, int x, int y)
{
	m_cursor.x = x;
	m_cursor.y = y;
	if ( state == GLUT_DOWN )
	{
		m_state = MOUSE_BUTTON_DOWN;
		m_stateClone = MOUSE_BUTTON_DOWN;
		m_down.x = x;
		m_down.y = y;
	}
	else if ( state == GLUT_UP )
	{
		m_state = MOUSE_BUTTON_UP;
		m_stateClone = MOUSE_BUTTON_UP;
		m_up.x = x;
		m_up.y = y;
	}
}

//------------------------------------------------------------------------------------------------------
// Name : Motion()
// Desc : }EX̃hbȌ
//------------------------------------------------------------------------------------------------------
void AsuraUserInterface::Motion(int x, int y)
{
	m_cursor.x = x;
	m_cursor.y = y;
	if ( m_focus )
	{
		m_delta.x += (x - m_down.x);
		m_delta.y += (y - m_down.y);
		m_down.x = x;
		m_down.y = y;
	}
}

//-------------------------------------------------------------------------------------------------------
// Name : PassiveMotion()
// Desc : }EẌړ̏
//-------------------------------------------------------------------------------------------------------
void AsuraUserInterface::PassiveMotion(int x, int y)
{
	m_cursor.x = x;
	m_cursor.y = y;
}

//----------------------------------------------------------------------------------------------------------
// Name : GetTextWidth()
// Desc : eLXg̕擾
//----------------------------------------------------------------------------------------------------------
int AsuraUserInterface::GetTextWidth(char *text)
{
	int width = 0;
	for ( int i=0; i<(int)strlen(text); i++ )
	{
		width += glutBitmapWidth(GLUT_BITMAP_HELVETICA_12, (int)text[i]);
	}
	width+=2;
	return width;
}

//---------------------------------------------------------------------------------------------------
// Name : DrawPartsFrame()
// Desc : g`
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsFrame(float x, float y, float w, float h, float depth)
{
	//@
	DrawRect(x-4, y, w, h, 0.0f, 0.0f, 0.0f, m_alpha, depth);

	//@Og
	glLineWidth(2.0f);
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	DrawRectLine(x-4, y, w, h, depth);
	glLineWidth(1.0f);
}

//---------------------------------------------------------------------------------------------------
// Name : DrawPartsText()
// Desc : s^uȂǂ`FbNɃeLXg`
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsText(int x, int y, char *text, int depth)
{
	if ( text == NULL )
		return;

	//@`Jnʒu
	glRasterPos3i(x, y, depth);

	//@eLXg`
	for ( int i=0; i<(int)strlen(text); i++ )
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, (int)text[i]);
}

//---------------------------------------------------------------------------------------------------
// Name : DrawPartsStaticText()
// Desc : s^u`FbNăeLXg`
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsStaticText(int x, int y, char *text, int depth)
{
	if ( text == NULL )
		return;

	int tx = x;
	int ty = y;

	//@̒܂Ń[v
	for ( int i=0; i<(int)strlen(text); i++)
	{
		//@'\n''\r's
		if ( text[i] == '\n' || text[i] == '\r' )
		{
			tx = x;
			ty += 14;
		}
		//@'\t'󔒂4
		if ( text[i] == '\t' )
		{
			tx += 16;
		}

		//@`ʒu
		glRasterPos3i(tx, ty, depth);

		//@`
		glutBitmapCharacter(GLUT_BITMAP_HELVETICA_12, (int)text[i]);
		
		//@1Ɉړ
		tx += glutBitmapWidth(GLUT_BITMAP_HELVETICA_12, (int)text[i]);
	}
}


//-----------------------------------------------------------------------------------------------------------
// Name : DrawPartsButton()
// Desc : {^`
//-----------------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsButton(int x, int y, int w, int h, int flag, int stroke)
{
	float bh = h /5.0f;

	switch ( flag )
	{
	//@ĂȂƂ
	case UI_NDOWN_AND_NOVERLAP:
		{
			DrawRect(x, y, w, bh,
				1.0f, 1.0f, 1.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x, y+bh*1, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				0.0f, 0.0f, 0.0f, m_alpha);
			
			DrawRect(x, y+bh*2, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha);

			DrawRect(x, y+bh*3, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x, y+bh*4, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				1.0f, 1.0f, 1.0f, m_alpha);

			glLineWidth(2.0f);
			glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
			DrawRectLine(x-1, y-1, w+2, h+2);
		}
		break;

	//@I[o[bv
	case UI_NDOWN_AND_OVERLAP:
		{
			DrawRect(x, y, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x, y+bh*1, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha, 
				1.0f, 1.0f, 1.0f, m_alpha);

			DrawRect(x, y+bh*2, w, bh,
				1.0, 1.0f, 1.0f, m_alpha);

			DrawRect(x, y+bh*3, w, bh,
				1.0f, 1.0f, 1.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x, y+bh*4, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				0.0f, 0.0f, 0.0f, m_alpha);

			glLineWidth(2.0f);
			glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
			DrawRectLine(x-1, y-1, w+2, h+2);
		}
		break;

	//@{^ꂽƂ
	case UI_DOWN_AND_OVERLAP:
		{
			DrawRect(x+stroke, y+stroke, w, bh,
				1.0f, 1.0f, 1.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x+stroke, y+stroke+bh*1, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				0.0f, 0.0f, 0.0f, m_alpha);

			DrawRect(x+stroke, y+stroke+bh*2, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha);

			DrawRect(x+stroke, y+stroke+bh*3, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x+stroke, y+stroke+bh*4, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				1.0f, 1.0f, 1.0f, m_alpha);

			glLineWidth(2.0f);
			glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
			DrawRectLine(x+stroke-1, y+stroke-1, w+2, h+2);
		}
		break;

	//@ftHg
	default:
		{
			DrawRect(x, y, w, bh,
				1.0f, 1.0f, 1.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x, y+bh*1, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				0.0f, 0.0f, 0.0f, m_alpha);
			
			DrawRect(x, y+bh*2, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha);

			DrawRect(x, y+bh*3, w, bh,
				0.0f, 0.0f, 0.0f, m_alpha,
				0.5f, 0.5f, 0.5f, m_alpha);

			DrawRect(x, y+bh*4, w, bh,
				0.5f, 0.5f, 0.5f, m_alpha,
				1.0f, 1.0f, 1.0f, m_alpha);

			glLineWidth(2.0f);
			glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
			DrawRectLine(x-1, y-1, w+2, h+2);
		}
		break;

	}

	//@ɖ߂
	glLineWidth(1.0f);
}

//----------------------------------------------------------------------------------------------------
// Name : DrawPartsHorizontalSlider()
// Desc : XC_[`
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsHorizontalSlider(int x, int y, int w, float pos)
{
	float s=10.0f;

	//@o[
	DrawRect(x, y+s-1, w, 2,
		1.0f, 1.0f, 1.0f, m_alpha);

	//@{^
	glBegin(GL_TRIANGLES);

	// 1
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x+pos, y, 0.0f);
	glVertex3f(x+pos-s, s+y, 0.0f);

	// 2
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	glVertex3f(x+pos, s+y, 0.0f);
	glVertex3f(x+pos, s+y, 0.0f);

	// 3
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x+pos-s, s+y, 0.0f);
	glVertex3f(x+pos, s*2.0f+y, 0.0f);

	// 4
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	glVertex3f(x+pos, s+y, 0.0f);

	// 5
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x+pos, s*2.0f+y, 0.0f);
	glVertex3f(x+pos+s, s+y, 0.0f);

	// 6
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	glVertex3f(x+pos, s+y, 0.0f);

	// 7
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x+pos+s, s+y, 0.0f);
	glVertex3f(x+pos, y, 0.0f);

	glEnd();
}

//----------------------------------------------------------------------------------------------------
// Name : DrawPartsVerticalSlider()
// Desc : XC_[`
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsVerticalSlider(int x, int y, int h, float pos)
{
	float s=10.0f;

	//@o[
	DrawRect(x-1, y, 2, h,
		1.0f, 1.0f, 1.0f, m_alpha);

	//@{^
	glBegin(GL_TRIANGLES);

	// 1
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x, y-s+pos, 0.0f);
	glVertex3f(x-s, y+pos, 0.0f);

	// 2
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	glVertex3f(x, y+pos, 0.0f);
	glVertex3f(x, y+pos, 0.0f);

	// 3
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x-s, y+pos, 0.0f);
	glVertex3f(x, y+s+pos, 0.0f);

	// 4
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	glVertex3f(x, y+pos, 0.0f);

	// 5
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x, y+s+pos, 0.0f);
	glVertex3f(x+s, y+pos, 0.0f);

	// 6
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	glVertex3f(x, y+pos, 0.0f);

	// 7
	glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
	glVertex3f(x+s, y+pos, 0.0f);
	glVertex3f(x, y-s+pos, 0.0f);

	glEnd();
}

//------------------------------------------------------------------------------------------------
// Name : DrawPartsCheckBox()
// Desc : `FbN{bNX`
//-------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsCheckBox(int x, int y, bool flag)
{
	//@g
	DrawRect(x, y, 16, 16,
		1.0f, 1.0f, 1.0f, m_alpha);

	//@`FbN邵
	if ( flag )
	{
		DrawRect(x+3, y+3, 10, 10,
			0.0f, 0.0f, 0.0f, m_alpha,
			0.3f, 0.3f, 0.3f, m_alpha);
	}
}

//----------------------------------------------------------------------------------------------------
// Name : DrawPartsListBox()
// Desc : Xg{bNX`
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsListBox(int x, int y, int w, int numItem, int curLine)
{
	int ch=20;
	
	//@݂̃C
	if ( curLine != -1 )
	{
		DrawRect(x, (y+2) +ch*curLine, w, ch*0.5f,
			1.0f, 1.0f, 1.0f, m_alpha,
			0.3f, 0.3f, 0.3f, m_alpha,
			1.0f);

		DrawRect(x, (y+2)+ ch*curLine + ch*0.5f, w, ch*0.5f,
			0.3f, 0.3f, 0.3f, m_alpha,
			1.0f, 1.0f, 1.0f, m_alpha,
			1.0f);
	}

	//@Ŝ̘g
	DrawRect(x, y+2, w, numItem*ch,
		0.5f, 0.5f, 0.5f, m_alpha, 1.0f);

	//@Og
	glLineWidth(2.0f);
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	DrawRectLine(x-1, y+1, w+2, numItem*ch+2);
	glLineWidth(1.0f);
}

//---------------------------------------------------------------------------------------------
// Name : DrawPartsComboBox()
// Desc : R{{bNX`
//---------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsComboBox(int x, int y, int w, int numItem, int curIndex, int flag)
{
	float bw = 12.0f;
	float bh = 20.0f;

	//@t[`
	DrawPartsFrame(x, y, w, bh);

	//@{^
	DrawPartsButton(x+w-1, y, 10, 20, flag);

	//@
	if ( flag == UI_DOWN_AND_OVERLAP )
	{
		glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w+1+2, y+6+2, 0.0f);
		glVertex3f(x+w+4+2, y+bh-5+2, 0.0f);
		glVertex3f(x+w+7+2, y+6+2, 0.0f);
		glEnd();
	}
	else if ( flag == UI_NDOWN_AND_OVERLAP )
	{
		glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w+1, y+6, 0.0f);
		glVertex3f(x+w+4, y+bh-5, 0.0f);
		glVertex3f(x+w+7, y+6, 0.0f);
		glEnd();
	}
	else if ( flag == UI_NDOWN_AND_NOVERLAP )
	{
		glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w+1, y+6, 0.0f);
		glVertex3f(x+w+4, y+bh-5, 0.0f);
		glVertex3f(x+w+7, y+6, 0.0f);
		glEnd();
	}
}

//----------------------------------------------------------------------------------------------------
// Name : DrawPartsSpinControl()
// Desc : XsRg[̃p[c`
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsSpinControl(int x, int y, int w, int upflag, int downflag)
{
	float bh= 20;

	//@t[`
	DrawPartsFrame(x, y, w, bh);

	//@
	if ( upflag == UI_NDOWN_AND_NOVERLAP )
	{
		glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w-1+5, y+2, 0.0f);
		glVertex3f(x+w-1+5-4, y+5, 0.0f);
		glVertex3f(x+w-1+5+4, y+5, 0.0f);
		glEnd();
	}
	else if ( upflag == UI_NDOWN_AND_OVERLAP )
	{
		glColor4f(0.0f, 0.0f, 0.0f, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w-1+5, y+2, 0.0f);
		glVertex3f(x+w-1+5-4, y+5, 0.0f);
		glVertex3f(x+w-1+5+4, y+5, 0.0f);
		glEnd();
	}
	else if ( upflag == UI_DOWN_AND_OVERLAP )
	{
		glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w-1+5+2, y+2+2, 0.0f);
		glVertex3f(x+w-1+5-4+2, y+5+2, 0.0f);
		glVertex3f(x+w-1+5+4+2, y+5+2, 0.0f);
		glEnd();
	}

	//@
	if ( downflag == UI_NDOWN_AND_NOVERLAP )
	{
		glColor4f(1.0f, 1.0f, 1.0, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w-1+5-4, y+12+2, 0.0f);
		glVertex3f(x+w-1+5, y+12+5, 0.0f);
		glVertex3f(x+w-1+5+4, y+12+2, 0.0f);
		glEnd();
	}
	else if ( downflag == UI_NDOWN_AND_OVERLAP )
	{
		glColor4f(0.0f, 0.0f, 0.0, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w-1+5-4, y+12+2, 0.0f);
		glVertex3f(x+w-1+5, y+12+5, 0.0f);
		glVertex3f(x+w-1+5+4, y+12+2, 0.0f);
		glEnd();
	}
	else if( downflag == UI_DOWN_AND_OVERLAP )
	{
		glColor4f(1.0f, 1.0f, 1.0, m_alpha);
		glBegin(GL_TRIANGLES);
		glVertex3f(x+w-1+5-4+2, y+12+2+2, 0.0f);
		glVertex3f(x+w-1+5+2, y+12+5+2, 0.0f);
		glVertex3f(x+w-1+5+4+2, y+12+2+2, 0.0f);
		glEnd();
	}
}

//----------------------------------------------------------------------------------------------------
// Name : DrawPartsGroupBox()
// Desc : O[v{bNXp̃p[c`
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsGroupBox(int x, int y, int w, int h, int textWidth)
{
	glLineWidth(1.0f);
	glColor4f(0.5f, 0.5f, 0.5f, m_alpha);
	glBegin(GL_LINE_STRIP);
	glVertex3f(x, y, 0.0f);
	glVertex3f(x, y+h, 0.0f);
	glVertex3f(x+w, y+h, 0.0f);
	glVertex3f(x+w, y, 0.0f);
	glEnd();

	glBegin(GL_LINES);
	glVertex3f(x, y, 0.0f);
	glVertex3f(x+8, y, 0.0f);
	glVertex3f(x+w, y, 0.0f);
	glVertex3f(x+textWidth, y, 0.0f);
	glEnd();

}

//----------------------------------------------------------------------------------------------------
// Name : DrawPartsGauge()
// Desc : Q[W̃p[c`
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DrawPartsGauge(int x, int y, int w, int h, int value, float *color)
{
	//@Q[W
	DrawRect(x, y+1, value, h*0.5f, 
		color[0]*0.5f, color[1]*0.5f, color[2]*0.5f, m_alpha,
		color[0]*1.0f, color[1]*1.0f, color[2]*1.0f, m_alpha);

	DrawRect(x, y+h*0.5f, value, h*0.5f,
		color[0]*1.0f, color[1]*1.0f, color[2]*1.0f, m_alpha,
		color[0]*0.5f, color[1]*0.5f, color[2]*0.5f, m_alpha);

	//@g
	glLineWidth(2.0f);
	glColor4f(1.0f, 1.0f, 1.0f, m_alpha);
	DrawRectLine(x-1, y, w+2, h);

	glLineWidth(1.0f);
}

//--------------------------------------------------------------------------------------------------
// Name : DoText()
// Desc : eLXg
//--------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoText(int x, int y, char *text)
{
	if ( text != NULL )
	{
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		DrawPartsText(x, y+14, text);
	}
}

//---------------------------------------------------------------------------------------------------
// Name : DoStaticText()
// Desc : X^eBbNeLXg
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoStaticText(int x, int y, char *text)
{
	if ( text != NULL )
	{
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		DrawPartsStaticText(x, y+14, text);
	}
}

//---------------------------------------------------------------------------------------------------
// Name : DoTextBox()
// Desc : eLXg{bNX
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoTextBox(int x, int y, int w, int h, char *text)
{
	if ( text != NULL )
	{
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		DrawPartsStaticText(x, y+14, text);
	}
	DrawPartsFrame(x, y, w, h);
}

//---------------------------------------------------------------------------------------------------
// Name : DoButton()
// Desc : {^̏
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoButton(int x, int y, int w, int h, char *text, bool &state)
{
	int flag = 0;
	
	//@͈̓`FbN
	if ( CheckRange(m_cursor.x, x, x+w) && CheckRange(m_cursor.y, y, y+h) )
	{
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			flag = UI_DOWN_AND_OVERLAP;
			state = true;
		}
		else
		{
			flag = UI_NDOWN_AND_OVERLAP;
			state = false;
		}
	}
	else if( !CheckRange(m_cursor.x, x, x+w) || !CheckRange(m_cursor.y, y, y+h ) )
	{
		flag = UI_NDOWN_AND_NOVERLAP;
		state = false;
	}

	//@{^`
	DrawPartsButton(x, y, w, h, flag);

	if ( text != NULL )
	{
		//@eLXg`
		float PosX = x+ (w*0.5+0.5) - (GetTextWidth(text)*0.5f+0.5);
		float PosY = 3.0f + y + h*0.5f;

		if ( flag == UI_DOWN_AND_OVERLAP )
		{
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			DrawPartsText(PosX+2, PosY+2, text);
		}
		else if ( flag == UI_NDOWN_AND_OVERLAP )
		{
			glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
			DrawPartsText(PosX, PosY, text);
		}
		else
		{
			glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
			DrawPartsText(PosX, PosY, text);
		}
	}

	if ( m_stateClone == MOUSE_BUTTON_DOWN && state)
	{
		m_stateClone = MOUSE_BUTTON_UP;
		state = true;
	}
	else
		state = false;

}

//-----------------------------------------------------------------------------------------------------------
// Name : DoRadioButton()
// Desc : WI{^̏
//-----------------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoRadioButton(int x, int y, int index, char *text, int &curIndex)
{
	bool state = false;

	//@IĂԂȂ`FbN
	if ( curIndex == index ) state = true;

	//@`FbN{bNX̕`
	DoCheckBox(x, y, text, state);

	//@`FbNĂCfbNXύX
	if ( state ) curIndex = index;
}

//------------------------------------------------------------------------------------------------------------
// Name : DoHorizontalSlider()
// Desc : XC_[̏
//------------------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoHorizontalSlider(int x, int y, int w, float min, float max, float &position)
{
	//@ۑ
	float tempPos = position;

	//@0`1͈̔͂ɂ
	float f = (position-min)/(max-min);
	
	//@͈̓`FbN
	if ( f < 0.0f ) f = 0.0f;
	if ( f > 1.0f ) f = 1.0f;
	
	//@l0`w͈̔͂ɂ
	float barPos = f * w;

	//@o[̔̏ꍇ
	if ( CheckRange(m_down.x, x+barPos-11, x+barPos+11 ) && CheckRange(m_down.y, y-1, y+21 ) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			m_focus = true;
			barPos = m_delta.x;
		}
	}
	else
	{
		//@o[̔O{^ĂȂꍇ
		if ( m_state == MOUSE_BUTTON_UP )
		{
			m_focus = false;
		}
		//@o[̔O{^Ăꍇ
		else if ( m_state == MOUSE_BUTTON_DOWN )
		{
			//@͈͓Ȃړ
			if ( m_focus && CheckRange(m_cursor.x, x-11, x+w+11) )
			{
				barPos = m_delta.x;
			}
		}
	}

	//@͈̓`FbN
	if ( 0 > barPos )
	{
		barPos = 0;
		m_delta.x = 0;
	}
	if ( w < barPos )
	{
		barPos = w;
		m_delta.x = w;
	}

	//@XC_[`
	DrawPartsHorizontalSlider(x, y, w, barPos);

	//@|WV߂
	if ( CheckRange(m_cursor.x, x-11, x+w+11 ) && m_state == MOUSE_BUTTON_DOWN )
		position = (max-min)/w * barPos + min;
	else
		position = tempPos;

	//@͈̓`FbN
	if ( position < min ) position = min;
	if ( position > max ) position = max;
}

//------------------------------------------------------------------------------------------------------------
// Name : DoVerticalSlider()
// Desc : XC_[̏
//------------------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoVerticalSlider(int x, int y, int h, float min, float max, float &position)
{
	//@ۑ
	float tempPos = position;

	//@0`1͈̔͂ɂ
	float f = (position-min)/(max-min);
	
	//@͈̓`FbN
	if ( f < 0.0f ) f = 0.0f;
	if ( f > 1.0f ) f = 1.0f;
	
	//@l0`h͈̔͂ɂ
	float barPos = f * h;

	//@o[̔̏ꍇ
	if ( CheckRange(m_down.x, x-11, x+11 ) && CheckRange(m_down.y, y-11+barPos, y+11+barPos ) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			m_focus = true;
			barPos = m_delta.y;
		}
	}
	else
	{
		//@o[̔O{^ĂȂꍇ
		if ( m_state == MOUSE_BUTTON_UP )
		{
			m_focus = false;
		}
		//@o[̔O{^Ăꍇ
		else if ( m_state == MOUSE_BUTTON_DOWN )
		{
			//@͈͓Ȃړ
			if ( m_focus && CheckRange(m_cursor.y, y-11, y+h+11) )
			{
				barPos = m_delta.y;
			}
		}
	}

	//@͈̓`FbN
	if ( 0 > barPos )
	{
		barPos = 0;
		m_delta.y = 0;
	}
	if ( h < barPos )
	{
		barPos = h;
		m_delta.y = h;
	}

	//@XC_[`
	DrawPartsVerticalSlider(x, y, h, barPos);

	//@|WV߂
	if ( CheckRange(m_cursor.y, y-11, y+h+11 ) && m_state == MOUSE_BUTTON_DOWN )
		position = (max-min)/h * barPos + min;
	else
		position = tempPos;

	//@͈̓`FbN
	if ( position < min ) position = min;
	if ( position > max ) position = max;
}


//----------------------------------------------------------------------------------------------------
// Name : DoCheckBox()
// Desc : `FbN{bNX̏
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoCheckBox(int x, int y, char *text, bool &state)
{
	//@Ă邩`FbN
	if ( CheckRange(m_down.x, x, x+16) && CheckRange(m_down.y, y, y+16) )
	{
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			if ( !m_focus && m_stateClone == MOUSE_BUTTON_DOWN )
				state = ( state ? false : true );
			m_state = MOUSE_BUTTON_UP;
		}
	}

	//@`FbN{bNX̕`
	DrawPartsCheckBox(x, y, state);

	//@eLXg`
	if ( text != NULL )
	{
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		DrawPartsText(x+20, y+12, text);
	}
}

//--------------------------------------------------------------------------------------------------------
// Name : DoListBox()
// Desc : Xg{bNX̏
//--------------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoListBox(int x, int y, int w, int numItem, char *text[], int &curLine)
{
	//@͈̓`FbN
	if ( CheckRange(m_cursor.x, x, x+w) && !m_focus )
	{
		//@{^Ă
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			//@ACe[v
			for ( int i=0; i<numItem; i++ )
			{
				//@IĂ郉CZo
				if ( CheckRange(m_cursor.y, y+22*(i+0), y+22*(i+1)) )
				{
					curLine = i;	//@IC
					m_state = MOUSE_BUTTON_UP;	//@}EX{^Abvɂ
					break;
				}
			}
		}
	}

	//@Xg{bNX̕`
	DrawPartsListBox(x, y, w, numItem, curLine);

	if ( text != NULL )
	{
		//@ACe`
		for ( int i=0; i<numItem; i++ )
		{
			//@IĂȂ甒F
			if ( i != curLine ) glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

			//@IĂԐF
			else glColor4f(1.0f, 0.0f, 0.0f, 1.0f);

			//@eLXg`
			DrawPartsText(x+4, y+(i*20)+16, text[i], 1.0f);
		}
	}
}

//---------------------------------------------------------------------------------------------------
// Name : DoComboBox()
// Desc : R{{bNX̏
//---------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoComboBox(int x, int y, int w, int numItem, char *text[], int &curIndex, bool &dispListBox)
{
	bool state = dispListBox;
	int flag = 0;

	//@͈̓`FbN
	if ( CheckRange(m_cursor.x, x+w-3, x+w+11) && CheckRange(m_cursor.y, y-1, y+20) )
	{
		//@{^ĂȂ
		if ( m_state == MOUSE_BUTTON_UP )
		{
			flag = UI_NDOWN_AND_OVERLAP;
		}
		//@{^ꂽ
		else if ( m_state == MOUSE_BUTTON_DOWN )
		{
			if ( CheckRange(m_down.x, x+w-3, x+w+11) && CheckRange(m_down.y, y-1, y+20) )
			{
				state = true;
				flag = UI_DOWN_AND_OVERLAP;
			}
			else
			{
				flag = UI_NDOWN_AND_OVERLAP;
			}
		}	
	}
	//@ĂȂ
	else
	{
		//@I[o[bv
		if ( m_state == MOUSE_BUTTON_UP 
			|| ( !CheckRange(m_cursor.x, x+w-3, x+w+11) || !CheckRange(m_cursor.y, y-1, y+20) ) )
		{
			flag = UI_NDOWN_AND_NOVERLAP;
		}
		//@͈͊Ołꂽ烊Xg{bNXB
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			if ( !CheckRange(m_down.x, x, x+w) 
				|| !CheckRange(m_down.y, y+12, y+22+numItem*20) )
				state = false;
		}
	}

	//@R{{bNX̃p[c`
	DrawPartsComboBox(x, y, w, numItem, curIndex, flag);

	//@eLXg`
	if ( text != NULL )
	{
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		DrawPartsText(x, y+14, text[curIndex]);
	}

	//@Xg{bNX̕`
	if ( state )
		DoListBox(x-4, y+22, w, numItem, text, curIndex);

	//@Ԃi[
	dispListBox = state;
}

//-----------------------------------------------------------------------------------------------------
// Name : DoSpinControl()
// Desc : XsRg[̏
//-----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoSpinControl(int x, int y, int w, float min, float max, float &value, float count)
{
	char text[20];
	bool upflag = false;
	bool downflag = false;
	int uiflag1 = 0;
	int uiflag2 = 0;

	//@͈͓̔̏ꍇ
	if ( CheckRange(m_cursor.x, x+w-2, x+w+10) && CheckRange(m_cursor.y, y-1, y+8) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			if ( CheckRange(m_down.x, x+w-2, x+w+10) && CheckRange(m_down.y, y-1, y+8) )
			{
				uiflag1 = UI_DOWN_AND_OVERLAP;
				upflag = true;
			}
			else
			{
				uiflag1 = UI_NDOWN_AND_OVERLAP;
				upflag = false;
			}
		}
		//@{^ꂽꍇ
		else
		{
			uiflag1 = UI_NDOWN_AND_OVERLAP;
			upflag = false;
		}
	}
	//@͈̔͊Ȍꍇ
	else
	{
		uiflag1 = UI_NDOWN_AND_NOVERLAP;
		upflag = false;
	}

	//@͈͓̔̏ꍇ
	if ( CheckRange(m_cursor.x, x+w-2, x+w+10) && CheckRange(m_cursor.y, y+11, y+20) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )	
		{
			if ( CheckRange(m_down.x, x+w-2, x+w+10) && CheckRange(m_down.y, y+11, y+20) )
			{
				uiflag2 = UI_DOWN_AND_OVERLAP;
				downflag = true;
			}
			else
			{
				uiflag2 = UI_NDOWN_AND_OVERLAP;
				downflag = false;
			}
		}
		//@{^ꂽꍇ
		else
		{
			uiflag2 = UI_NDOWN_AND_OVERLAP;
			downflag = false;
		}
	}
	//@͈̔͊Ȍꍇ
	else
	{
		uiflag2 = UI_NDOWN_AND_NOVERLAP;
		downflag = false;
	}

	//@͈͐Ɉ|ĂȂ`FbN
	if ( value > max ) value = max;
	if ( value < min ) value = min;
	
	//@l𕶎ɕϊ
	sprintf(text, "%.3f", value);

	//@{^
	DrawPartsButton(x+w-1, y, 10, 8, uiflag1);
	DrawPartsButton(x+w-1, y+12, 10, 8, uiflag2);

	//@p[c`
	DrawPartsSpinControl(x, y, w, uiflag1, uiflag2);

	//@eLXg`
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	DrawPartsText(x, y+14, text);
	
	//@JEg
	if ( upflag && m_stateClone == MOUSE_BUTTON_DOWN )	value += count;
	if ( downflag && m_stateClone == MOUSE_BUTTON_DOWN )	value -= count;

	//@͈̓`FbN
	if ( value > max ) value = max;
	if ( value < min ) value = min;

	//@1mɃJEg邽߂Ƀ{^̏ԂύX
	if ( (upflag || downflag) && m_stateClone == MOUSE_BUTTON_DOWN )
		m_stateClone = MOUSE_BUTTON_UP;
}

//----------------------------------------------------------------------------------------------------
// Name : DoSpinControl()
// Desc : XsRg[̏
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoSpinControl(int x, int y, int w, double min, double max, double &value, double count)
{
	char text[20];
	bool upflag = false;
	bool downflag = false;
	int uiflag1 = 0;
	int uiflag2 = 0;

	//@͈͓̔̏ꍇ
	if ( CheckRange(m_cursor.x, x+w-2, x+w+10) && CheckRange(m_cursor.y, y-1, y+8) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			if ( CheckRange(m_down.x, x+w-2, x+w+10) && CheckRange(m_down.y, y-1, y+8) )
			{
				uiflag1 = UI_DOWN_AND_OVERLAP;
				upflag = true;
			}
			else
			{
				uiflag1 = UI_NDOWN_AND_OVERLAP;
				upflag = false;
			}
		}
		//@{^ꂽꍇ
		else
		{
			uiflag1 = UI_NDOWN_AND_OVERLAP;
			upflag = false;
		}
	}
	//@͈̔͊Ȍꍇ
	else
	{
		uiflag1 = UI_NDOWN_AND_NOVERLAP;
		upflag = false;
	}

	//@͈͓̔̏ꍇ
	if ( CheckRange(m_cursor.x, x+w-2, x+w+10) && CheckRange(m_cursor.y, y+11, y+20) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )	
		{
			if ( CheckRange(m_down.x, x+w-2, x+w+10) && CheckRange(m_down.y, y+11, y+20) )
			{
				uiflag2 = UI_DOWN_AND_OVERLAP;
				downflag = true;
			}
			else
			{
				uiflag2 = UI_NDOWN_AND_OVERLAP;
				downflag = false;
			}
		}
		//@{^ꂽꍇ
		else
		{
			uiflag2 = UI_NDOWN_AND_OVERLAP;
			downflag = false;
		}
	}
	//@͈̔͊Ȍꍇ
	else
	{
		uiflag2 = UI_NDOWN_AND_NOVERLAP;
		downflag = false;
	}

	//@͈͐Ɉ|ĂȂ`FbN
	if ( value > max ) value = max;
	if ( value < min ) value = min;
	
	//@l𕶎ɕϊ
	sprintf(text, "%.3f", value);

	//@{^
	DrawPartsButton(x+w-1, y, 10, 8, uiflag1);
	DrawPartsButton(x+w-1, y+12, 10, 8, uiflag2);

	//@p[c`
	DrawPartsSpinControl(x, y, w, uiflag1, uiflag2);

	//@eLXg`
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	DrawPartsText(x, y+14, text);
	
	//@JEg
	if ( upflag && m_stateClone == MOUSE_BUTTON_DOWN )	value += count;
	if ( downflag && m_stateClone == MOUSE_BUTTON_DOWN )	value -= count;

	//@͈̓`FbN
	if ( value > max ) value = max;
	if ( value < min ) value = min;

	//@1mɃJEg邽߂Ƀ{^̏ԂύX
	if ( ( upflag || downflag ) && m_stateClone == MOUSE_BUTTON_DOWN )
		m_stateClone = MOUSE_BUTTON_UP;
}

//----------------------------------------------------------------------------------------------------
// Name : DoSpinControl()
// Desc : XsRg[̏
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoSpinControl(int x, int y, int w, int min, int max, int &value, int count)
{
	char text[20];
	bool upflag = false;
	bool downflag = false;
	int uiflag1 = 0;
	int uiflag2 = 0;

	//@͈͓̔̏ꍇ
	if ( CheckRange(m_cursor.x, x+w-2, x+w+10) && CheckRange(m_cursor.y, y-1, y+8) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )
		{
			if ( CheckRange(m_down.x, x+w-2, x+w+10) && CheckRange(m_down.y, y-1, y+8) )
			{
				uiflag1 = UI_DOWN_AND_OVERLAP;
				upflag = true;
			}
			else
			{
				uiflag1 = UI_NDOWN_AND_OVERLAP;
				upflag = false;
			}
		}
		//@{^ꂽꍇ
		else
		{
			uiflag1 = UI_NDOWN_AND_OVERLAP;
			upflag = false;
		}
	}
	//@͈̔͊Ȍꍇ
	else
	{
		uiflag1 = UI_NDOWN_AND_NOVERLAP;
		upflag = false;
	}

	//@͈͓̔̏ꍇ
	if ( CheckRange(m_cursor.x, x+w-2, x+w+10) && CheckRange(m_cursor.y, y+11, y+20) )
	{
		//@{^Ăꍇ
		if ( m_state == MOUSE_BUTTON_DOWN )	
		{
			if ( CheckRange(m_down.x, x+w-2, x+w+10) && CheckRange(m_down.y, y+11, y+20) )
			{
				uiflag2 = UI_DOWN_AND_OVERLAP;
				downflag = true;
			}
			else
			{
				uiflag2 = UI_NDOWN_AND_OVERLAP;
				downflag = false;
			}
		}
		//@{^ꂽꍇ
		else
		{
			uiflag2 = UI_NDOWN_AND_OVERLAP;
			downflag = false;
		}
	}
	//@͈̔͊Ȍꍇ
	else
	{
		uiflag2 = UI_NDOWN_AND_NOVERLAP;
		downflag = false;
	}

	//@͈͐Ɉ|ĂȂ`FbN
	if ( value > max ) value = max;
	if ( value < min ) value = min;
	
	//@l𕶎ɕϊ
	sprintf(text, "%d", value);

	//@{^
	DrawPartsButton(x+w-1, y, 10, 8, uiflag1);
	DrawPartsButton(x+w-1, y+12, 10, 8, uiflag2);

	//@p[c`
	DrawPartsSpinControl(x, y, w, uiflag1, uiflag2);

	//@eLXg`
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	DrawPartsText(x, y+14, text);
	
	//@JEg
	if ( upflag && m_stateClone == MOUSE_BUTTON_DOWN )	value += count;
	if ( downflag && m_stateClone == MOUSE_BUTTON_DOWN )	value -= count;

	//@͈̓`FbN
	if ( value > max ) value = max;
	if ( value < min ) value = min;

	//@1JEg邽߂Ƀ{^̏ԂύX
	if ( (upflag || downflag) && m_stateClone == MOUSE_BUTTON_DOWN )
		m_stateClone = MOUSE_BUTTON_UP;
}

//------------------------------------------------------------------------------------------------------
// Name : DoGroupBox()
// Desc : O[v{bNX̏
//------------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoGroupBox(int x, int y, int w, int h, char *caption)
{
	int tw = 0;
	if ( caption != NULL )
	{
		//@eLXg̕擾
		tw = GetTextWidth(caption);
	
		//@LvV
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
		DrawPartsText(x+8, y+6, caption);
	}

	//@g`
	DrawPartsGroupBox(x, y, w, h, tw);
}

//----------------------------------------------------------------------------------------------------
// Name : DoGauge()
// Desc : Q[W̏
//----------------------------------------------------------------------------------------------------
void AsuraUserInterface::DoGauage(int x, int y, int w, int h, int min, int max, int value, float *color)
{
	float pos = (float)(value-min)/(float)(max-min);

	if ( pos < 0.0f ) pos = 0.0f;
	if ( pos > 1.0f ) pos = 1.0f;
	pos *= w;

	if ( pos < 0.0f ) pos = 0.0f;
	if ( pos > w ) pos = w;

	DrawPartsGauge(x, y, w, h, pos, color);
}

}	//@namespcae Asura